"use strict";

exports.__esModule = true;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var StateStorage = function () {
  function StateStorage(farce, namespace) {
    _classCallCheck(this, StateStorage);

    this._getFallbackLocationKey = farce.createHref;
    this._stateKeyPrefix = namespace + "|";
  }

  StateStorage.prototype.read = function read(location, key) {
    var stateKey = this._getStateKey(location, key);

    try {
      var value = window.sessionStorage.getItem(stateKey);
      // === null is probably sufficient.
      if (value == null) {
        return undefined;
      }

      // We want to catch JSON parse errors in case someone separately threw
      // junk into sessionStorage under our namespace.
      return JSON.parse(value);
    } catch (e) {
      // Pretend that the entry doesn't exist.
      return undefined;
    }
  };

  StateStorage.prototype.save = function save(location, key, value) {
    var stateKey = this._getStateKey(location, key);

    if (value === undefined) {
      try {
        window.sessionStorage.removeItem(stateKey);
      } catch (e) {
        // No need to handle errors here.
      }

      return;
    }

    // Unlike with read, we want to fail on invalid values here, since the
    // value here is provided by the caller of this method.
    var valueString = JSON.stringify(value);

    try {
      window.sessionStorage.setItem(stateKey, valueString);
    } catch (e) {
      // No need to handle errors here either. If it didn't work, it didn't
      // work. We make no guarantees about actually saving the value.
    }
  };

  StateStorage.prototype._getStateKey = function _getStateKey(location, key) {
    var locationKey = location.key || this._getFallbackLocationKey(location);
    var stateKeyBase = "" + this._stateKeyPrefix + locationKey;
    return key == null ? stateKeyBase : stateKeyBase + "|" + key;
  };

  return StateStorage;
}();

exports.default = StateStorage;
module.exports = exports["default"];